/*
 * MFVec3f.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h> 
#include <assert.h>
#include "stdafx.h"

#include "MFVec3f.h"
#include "SFVec3f.h"
#include "Vec3f.h"

MFVec3f::MFVec3f() : MFFloat(3)
{
}

MFVec3f::MFVec3f(const float values[], int len) : MFFloat(values, len, 3)
{
}

MFVec3f::MFVec3f(const Vec3f *values, int len)
{
    _value.resize(len * 3);
    for (int i = 0; i < len; i++) {
	_value[i*3] = values[i].x;
	_value[i*3+1] = values[i].y;
	_value[i*3+2] = values[i].z;
    }
}

bool
MFVec3f::equals(const FieldValue *value) const
{
    return value->getType() == MFVEC3F && MFFloat::equals((const MFFloat *) value);
}

FieldValue *
MFVec3f::getSFValue(int index) const
{
    return new SFVec3f(getValue(index));
}

void
MFVec3f::setSFValue(int index, FieldValue *value)
{
    if (value->getType() != SFVEC3F) {
	assert(0);
	return;
    }

    const float	*values = ((SFVec3f *) value)->getValue();

    _value[index*3] = values[0];
    _value[index*3+1] = values[1];
    _value[index*3+2] = values[2];
}
