/*
 * MainWindow.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <errno.h>
#include <stdio.h>

#include "stdafx.h"
#include "MainWindow.h"
#include "swt.h"
#include "resource.h"

#include "Scene.h"
#include "Node.h"
#include "Path.h"
#include "MFFloat.h"
#include "MFNode.h"
#include "MFString.h"
#include "SFInt32.h"
#include "SFNode.h"
#include "SceneTreeView.h"
#include "Scene3DView.h"
#include "FieldView.h"
#include "SceneGraphView.h"
#include "ChannelView.h"
#include "ToolbarWindow.h"
#include "DuneApp.h"
#include "MoveCommand.h"
#include "URL.h"
#include "ElevationGridDialog.h"
#include "PreferencesDialog.h"
#include "PreviewSettingsDialog.h"

#define ARRAYSIZE(A) (sizeof(A) / sizeof(A[0]))
#define BS -1

static int standardButtons[] = {
    0,  ID_FILE_NEW,
    1,  ID_FILE_OPEN,
    2,  ID_FILE_SAVE,
    3,  ID_FILE_PREVIEW,
    BS, 0,
    4,  ID_EDIT_CUT,
    5,  ID_EDIT_COPY,
    6,  ID_EDIT_PASTE,
    7,  ID_EDIT_DELETE,
    BS, 0,
    8,  ID_MOVE_MODE,
    9,  ID_ROTATE_MODE,
    10, ID_SCALE_MODE,
};

static int vcrButtons[] = {
    0,  ID_REWIND,
    1,  ID_STOP,
    2,  ID_RECORD,
    3,  ID_PLAY,
    4,  ID_FAST_FORWARD,
};

static int buttons1[] = {
    NODE_BOX,			ID_NEW_BOX,
    NODE_SPHERE,		ID_NEW_SPHERE,
    NODE_CONE,			ID_NEW_CONE,
    NODE_CYLINDER,		ID_NEW_CYLINDER,
    NODE_INDEXED_FACE_SET,	ID_NEW_INDEXED_FACE_SET,
    NODE_INDEXED_LINE_SET,	ID_NEW_INDEXED_LINE_SET,
    NODE_POINT_SET,		ID_NEW_POINT_SET,
    NODE_ELEVATION_GRID,	ID_NEW_ELEVATION_GRID,
    NODE_EXTRUSION,		ID_NEW_EXTRUSION,
    NODE_TEXT,			ID_NEW_TEXT,
    BS,				0,
    NODE_GROUP,			ID_NEW_GROUP,
    NODE_TRANSFORM,		ID_NEW_TRANSFORM,
    NODE_ANCHOR,		ID_NEW_ANCHOR,
    NODE_BILLBOARD,		ID_NEW_BILLBOARD,
    NODE_COLLISION,		ID_NEW_COLLISION,
    NODE_INLINE,		ID_NEW_INLINE,
    NODE_LOD,			ID_NEW_LOD,
    NODE_SWITCH,		ID_NEW_SWITCH,
};

static int buttons2[] = {
    NODE_SHAPE,			ID_NEW_SHAPE,
    NODE_COORDINATE,		ID_NEW_COORDINATE,
    NODE_NORMAL,		ID_NEW_NORMAL,
    NODE_COLOR,			ID_NEW_COLOR,
    NODE_TEXTURE_COORDINATE,	ID_NEW_TEXTURE_COORDINATE,
    BS,				0,
    NODE_APPEARANCE,		ID_NEW_APPEARANCE,
    NODE_MATERIAL,		ID_NEW_MATERIAL,
    NODE_IMAGE_TEXTURE,		ID_NEW_IMAGE_TEXTURE,
    NODE_PIXEL_TEXTURE,		ID_NEW_PIXEL_TEXTURE,
    NODE_MOVIE_TEXTURE,		ID_NEW_MOVIE_TEXTURE,
    NODE_TEXTURE_TRANSFORM,	ID_NEW_TEXTURE_TRANSFORM,
    NODE_FONT_STYLE,		ID_NEW_FONT_STYLE,
    BS,				0,
    NODE_SOUND,			ID_NEW_SOUND,
    NODE_AUDIO_CLIP,		ID_NEW_AUDIO_CLIP,
    BS,				0,
    NODE_DIRECTIONAL_LIGHT,	ID_NEW_DIRECTIONAL_LIGHT,
    NODE_POINT_LIGHT,		ID_NEW_POINT_LIGHT,
    NODE_SPOT_LIGHT,		ID_NEW_SPOT_LIGHT,
    BS,				0,
    NODE_FOG,			ID_NEW_FOG,
    NODE_BACKGROUND,		ID_NEW_BACKGROUND,
    NODE_VIEWPOINT,		ID_NEW_VIEWPOINT,
    NODE_NAVIGATION_INFO,	ID_NEW_NAVIGATION_INFO,
    NODE_WORLD_INFO,		ID_NEW_WORLD_INFO
};

static int buttons3[] = {
    NODE_PROXIMITY_SENSOR,	ID_NEW_PROXIMITY_SENSOR,
    NODE_CYLINDER_SENSOR,	ID_NEW_CYLINDER_SENSOR,
    NODE_SPHERE_SENSOR,		ID_NEW_SPHERE_SENSOR,
    NODE_PLANE_SENSOR,		ID_NEW_PLANE_SENSOR,
    NODE_TIME_SENSOR,		ID_NEW_TIME_SENSOR,
    NODE_TOUCH_SENSOR,		ID_NEW_TOUCH_SENSOR,
    NODE_VISIBILITY_SENSOR,	ID_NEW_VISIBILITY_SENSOR,
    BS,				0,
    NODE_COLOR_INTERPOLATOR,	ID_NEW_COLOR_INTERPOLATOR,
    NODE_COORDINATE_INTERPOLATOR,ID_NEW_COORDINATE_INTERPOLATOR,
    NODE_POSITION_INTERPOLATOR,	ID_NEW_POSITION_INTERPOLATOR,
    NODE_ORIENTATION_INTERPOLATOR,ID_NEW_ORIENTATION_INTERPOLATOR,
    NODE_NORMAL_INTERPOLATOR,	ID_NEW_NORMAL_INTERPOLATOR,
    NODE_SCALAR_INTERPOLATOR,	ID_NEW_SCALAR_INTERPOLATOR,
    BS,				0,
    NODE_SCRIPT,		ID_NEW_SCRIPT
};

static int
timerCB(void *data)
{
    return ((MainWindow *) data)->OnTimer();
}

MainWindow::MainWindow(Scene *scene, SWND wnd)
  : PanedWindow(scene, wnd, false)
{
    _timer = NULL;
    _clipboardNode = NULL;

    _menu = swLoadMenuBar(wnd, IDR_DUNETYPE);
    RefreshRecentFileMenu();

    SWND i = swCreateCanvas("", 0, 0, 800, 400, wnd);
    _innerPane = new PanedWindow(scene, i, true);

    SWND t = swCreateCanvas("", 0, 0, 800, 100, wnd);
    _toolbarWindow = new ToolbarWindow(scene, t, this);

    SWND t2 = swCreateCanvas("", 0, 0, 800, 100, wnd);
    _toolbarWindow2 = new ToolbarWindow(scene, t2, this);

    SWND c1 = swCreateCanvas("", 0, 0, 200, 400, i);
    _treeView = new SceneTreeView(scene, c1);
    if (TheApp->GetBoolPreference("ShowSceneTree", true)) {
	_innerPane->SetPane(_treeView, PW_LEFT);
	swMenuSetFlags(_menu, ID_VIEW_SCENE_TREE, SW_MENU_CHECKED, SW_MENU_CHECKED);
    } else {
	swHideWindow(_treeView->GetWindow());
    }

    SWND c2 = swCreateCanvas("", 200, 0, 400, 400, i);
    _3DView = new Scene3DView(scene, c2);
    _innerPane->SetPane(_3DView, PW_CENTER);

    SWND c3 = swCreateCanvas("", 600, 0, 270, 200, i);
    _fieldView = new FieldView(scene, c3);
    if (TheApp->GetBoolPreference("ShowFieldView", true))
    {
	_innerPane->SetPane(_fieldView, PW_RIGHT);
	swMenuSetFlags(_menu, ID_VIEW_FIELD_VIEW, SW_MENU_CHECKED, SW_MENU_CHECKED);
    } else {
	swHideWindow(_fieldView->GetWindow());
    }

    SWND c4 = swCreateCanvas("", 0, 0, 800, 160, i);
    _graphView = new SceneGraphView(scene, c4);
    if (TheApp->GetBoolPreference("ShowRouteView", false)) {
	_innerPane->SetPane(_graphView, PW_TOP);
	swMenuSetFlags(_menu, ID_VIEW_SCENE_GRAPH, SW_MENU_CHECKED, SW_MENU_CHECKED);
    } else {
	swHideWindow(_graphView->GetWindow());
    }

    SWND c5 = swCreateCanvas("", 0, 400, 800, 200, i);
    _channelView = new ChannelView(scene, c5);
    if (TheApp->GetBoolPreference("ShowChannelView", true)) {
	_innerPane->SetPane(_channelView, PW_BOTTOM);
	swMenuSetFlags(_menu, ID_VIEW_CHANNEL_VIEW, SW_MENU_CHECKED, SW_MENU_CHECKED);
    } else {
	swHideWindow(_channelView->GetWindow());
    }

    _standardToolbar = LoadToolbar(_toolbarWindow, IDB_STANDARD_TOOLBAR,
				   ARRAYSIZE(standardButtons) / 2,
				   standardButtons, ID_VIEW_TOOLBAR,
				   "StandardToolbar");
    _nodeToolbar1 = LoadToolbar(_toolbarWindow, IDB_NODE_ICONS,
				ARRAYSIZE(buttons1) / 2, buttons1,
				ID_VIEW_NODE_TOOLBAR_1, "NodeToolbar1");
    _nodeToolbar2 = LoadToolbar(_toolbarWindow, IDB_NODE_ICONS,
				ARRAYSIZE(buttons2) / 2, buttons2,
				ID_VIEW_NODE_TOOLBAR_2, "NodeToolbar2");
    _nodeToolbar3 = LoadToolbar(_toolbarWindow, IDB_NODE_ICONS,
    				ARRAYSIZE(buttons3) / 2,
				buttons3, ID_VIEW_NODE_TOOLBAR_3,
				"NodeToolbar3");
    _vcrToolbar = LoadToolbar(_toolbarWindow2, IDB_VCR_TOOLBAR,
			      ARRAYSIZE(vcrButtons) / 2, vcrButtons,
			      ID_VIEW_PLAY_TOOLBAR, "PlayToolbar");
    SetPane(_innerPane, PW_CENTER);
    SetPane(_toolbarWindow, PW_TOP);
    SetPane(_toolbarWindow2, PW_BOTTOM);
    swMenuSetFlags(_menu, ID_MOVE_MODE, SW_MENU_RADIO_ITEM, SW_MENU_RADIO_ITEM);
    swMenuSetFlags(_menu, ID_ROTATE_MODE, SW_MENU_RADIO_ITEM, SW_MENU_RADIO_ITEM);
    swMenuSetFlags(_menu, ID_SCALE_MODE, SW_MENU_RADIO_ITEM, SW_MENU_RADIO_ITEM);
    SetTransformMode(TM_TRANSLATE);
    scene->setSelection(scene->getRoot());
 
    Layout();
    UpdateTitle();
    if (TheApp->GetBoolPreference("MaximizeWindows", false)) {
	swMaximizeWindow(_wnd);
    }
    swShowWindow(_wnd);
    scene->UpdateViews(NULL, UPDATE_ALL, NULL);
}

MainWindow::~MainWindow()
{
    Stop();
    TheApp->SetBoolPreference("MaximizeWindows", swIsMaximized(_wnd) != 0);
    delete _treeView;
    delete _3DView;
    delete _fieldView;
    delete _graphView;
    delete _channelView;
    delete _toolbarWindow;
    delete _toolbarWindow2;
    delete _innerPane;
    delete _scene;
}

void
MainWindow::UpdateTitle()
{
    String	title = "Dune: ";
    const char *path = _scene->getPath();

    if (path[0]) {
	title += path;
    } else {
	title += "Untitled";
    }

    swSetTitle(_wnd, title);
}

void
MainWindow::OnCommand(int id)
{
    switch (id) {
      case ID_FILE_OPEN:
	OnFileOpen();
	break;
      case ID_FILE_NEW:
	TheApp->OnFileNew();
	break;
      case ID_FILE_PREVIEW:
	OnFilePreview();
	break;
      case ID_FILE_SAVE:
	OnFileSave();
	break;
      case ID_FILE_SAVE_AS:
	OnFileSaveAs();
	break;
      case ID_FILE_CLOSE:
	TheApp->OnFileClose(this);
	break;
      case ID_FILE_MRU_FILE1:
      case ID_FILE_MRU_FILE2:
      case ID_FILE_MRU_FILE3:
      case ID_FILE_MRU_FILE4:
	OpenFileCheck(TheApp->GetRecentFile(id - ID_FILE_MRU_FILE1));
	break;
      case ID_APP_EXIT:
	TheApp->OnFileExit();
	break;
      case ID_EDIT_CUT:
	OnEditCut();
	break;
      case ID_EDIT_COPY:
	OnEditCopy();
	break;
      case ID_EDIT_PASTE:
	OnEditPaste();
	break;
      case ID_EDIT_DELETE:
	_scene->DeleteSelected();
	break;
      case ID_EDIT_UNDO:
	if (_scene->canUndo()) _scene->undo();
	break;
      case ID_EDIT_REDO:
	if (_scene->canRedo()) _scene->redo();
	break;
      case ID_MOVE_MODE:
	SetTransformMode(TM_TRANSLATE);
	break;
      case ID_SCALE_MODE:
	SetTransformMode(TM_SCALE);
	break;
      case ID_ROTATE_MODE:
	SetTransformMode(TM_ROTATE);
	break;
      case ID_VIEW_SCENE_TREE:
	ToggleView(PW_LEFT, _treeView, ID_VIEW_SCENE_TREE, "ShowSceneTree");
	break;
      case ID_VIEW_FIELD_VIEW:
	ToggleView(PW_RIGHT, _fieldView, ID_VIEW_FIELD_VIEW, "ShowFieldView");
	break;
      case ID_VIEW_SCENE_GRAPH:
	ToggleView(PW_TOP, _graphView, ID_VIEW_SCENE_GRAPH, "ShowRouteView");
	break;
      case ID_VIEW_CHANNEL_VIEW:
	ToggleView(PW_BOTTOM, _channelView, ID_VIEW_CHANNEL_VIEW,
		   "ShowChannelView");
	break;
      case ID_VIEW_TOOLBAR:
	ToggleToolbar(_toolbarWindow, _standardToolbar, ID_VIEW_TOOLBAR,
		      "StandardToolbar");
	break;
      case ID_VIEW_NODE_TOOLBAR_1:
	ToggleToolbar(_toolbarWindow, _nodeToolbar1, ID_VIEW_NODE_TOOLBAR_1,
		      "NodeToolbar1");
	break;
      case ID_VIEW_NODE_TOOLBAR_2:
	ToggleToolbar(_toolbarWindow, _nodeToolbar2, ID_VIEW_NODE_TOOLBAR_2,
		      "NodeToolbar2");
	break;
      case ID_VIEW_NODE_TOOLBAR_3:
	ToggleToolbar(_toolbarWindow, _nodeToolbar3, ID_VIEW_NODE_TOOLBAR_3,
		      "NodeToolbar3");
	break;
      case ID_VIEW_PLAY_TOOLBAR:
	ToggleToolbar(_toolbarWindow2, _vcrToolbar, ID_VIEW_PLAY_TOOLBAR,
		      "PlayToolbar");
	break;

      case ID_NEW_ANCHOR:
	CreateNode("Anchor");
	break;
      case ID_NEW_APPEARANCE:
	InsertNode(NODE_APPEARANCE, "Appearance");
	break;
      case ID_NEW_AUDIO_CLIP:
	InsertNode(NODE_AUDIO_CLIP, "AudioClip");
	break;
      case ID_NEW_BACKGROUND:
	CreateNode("Background");
	break;
      case ID_NEW_BILLBOARD:
	CreateNode("Billboard");
	break;
      case ID_NEW_BOX:
	CreateGeometryNode("Box");
	break;
      case ID_NEW_COLLISION:
	CreateNode("Collision");
	break;
      case ID_NEW_COLOR:
	InsertNode(NODE_COLOR, "Color");
	break;
      case ID_NEW_COLOR_INTERPOLATOR:
	CreateNode("ColorInterpolator");
	break;
      case ID_NEW_CONE:
	CreateGeometryNode("Cone");
	break;
      case ID_NEW_COORDINATE:
	InsertNode(NODE_COORDINATE, "Coordinate");
	break;
      case ID_NEW_COORDINATE_INTERPOLATOR:
	CreateNode("CoordinateInterpolator");
	break;
      case ID_NEW_CYLINDER:
	CreateGeometryNode("Cylinder");
	break;
      case ID_NEW_CYLINDER_SENSOR:
	CreateNode("CylinderSensor");
	break;
      case ID_NEW_DIRECTIONAL_LIGHT:
	CreateNode("DirectionalLight");
	break;
      case ID_NEW_ELEVATION_GRID:
	CreateElevationGrid();
	break;
      case ID_NEW_EXTRUSION:
	CreateGeometryNode("Extrusion");
	break;
      case ID_NEW_FOG:
	CreateNode("Fog");
	break;
      case ID_NEW_FONT_STYLE:
	InsertNode(NODE_FONT_STYLE, "FontStyle");
	break;
      case ID_NEW_GROUP:
	CreateNode("Group");
	break;
      case ID_NEW_IMAGE_TEXTURE:
	InsertImageTexture();
	break;
      case ID_NEW_INDEXED_FACE_SET:
	CreateGeometryNode("IndexedFaceSet");
	break;
      case ID_NEW_INDEXED_LINE_SET:
	CreateGeometryNode("IndexedLineSet");
	break;
      case ID_NEW_INLINE:
	CreateNode("Inline");
	break;
      case ID_NEW_LOD:
	CreateNode("LOD");
	break;
      case ID_NEW_MATERIAL:
	InsertNode(NODE_MATERIAL, "Material");
	break;
      case ID_NEW_MOVIE_TEXTURE:
	InsertNode(NODE_MOVIE_TEXTURE, "MovieTexture");
	break;
      case ID_NEW_NAVIGATION_INFO:
	CreateNode("NavigationInfo");
	break;
      case ID_NEW_NORMAL:
	InsertNode(NODE_NORMAL, "Normal");
	break;
      case ID_NEW_NORMAL_INTERPOLATOR:
	CreateNode("NormalInterpolator");
	break;
      case ID_NEW_ORIENTATION_INTERPOLATOR:
	CreateNode("OrientationInterpolator");
	break;
      case ID_NEW_PIXEL_TEXTURE:
	InsertNode(NODE_TEXTURE, "PixelTexture");
	break;
      case ID_NEW_PLANE_SENSOR:
	CreateNode("PlaneSensor");
	break;
      case ID_NEW_POINT_LIGHT:
	CreateNode("PointLight");
	break;
      case ID_NEW_POINT_SET:
	CreateGeometryNode("PointSet");
	break;
      case ID_NEW_POSITION_INTERPOLATOR:
	CreateNode("PositionInterpolator");
	break;
      case ID_NEW_PROXIMITY_SENSOR:
	CreateNode("ProximitySensor");
	break;
      case ID_NEW_SCALAR_INTERPOLATOR:
	CreateNode("ScalarInterpolator");
	break;
      case ID_NEW_SCRIPT:
//	CreateNode("Script");
	break;
      case ID_NEW_SHAPE:
	CreateNode("Shape");
	break;
      case ID_NEW_SOUND:
	CreateNode("Sound");
	break;
      case ID_NEW_SPHERE:
	CreateGeometryNode("Sphere");
	break;
      case ID_NEW_SPHERE_SENSOR:
	CreateNode("SphereSensor");
	break;
      case ID_NEW_SPOT_LIGHT:
	CreateNode("SpotLight");
	break;
      case ID_NEW_SWITCH:
	CreateNode("Switch");
	break;
      case ID_NEW_TEXT:
	CreateGeometryNode("Text");
	break;
      case ID_NEW_TEXTURE_COORDINATE:
	InsertNode(NODE_TEXTURE_COORDINATE, "TextureCoordinate");
	break;
      case ID_NEW_TEXTURE_TRANSFORM:
	InsertNode(NODE_TEXTURE_TRANSFORM, "TextureTransform");
	break;
      case ID_NEW_TIME_SENSOR:
	CreateNode("TimeSensor");
	break;
      case ID_NEW_TOUCH_SENSOR:
	CreateNode("TouchSensor");
	break;
      case ID_NEW_TRANSFORM:
	CreateNode("Transform");
	break;
      case ID_NEW_VIEWPOINT:
	CreateNode("Viewpoint");
	break;
      case ID_NEW_VISIBILITY_SENSOR:
	CreateNode("VisibilitySensor");
	break;
      case ID_NEW_WORLD_INFO:
	CreateNode("WorldInfo");
	break;
      case ID_PLAY:
	Play();
	break;
      case ID_STOP:
	Stop();
	break;
      case ID_RECORD:
	Record();
	break;
      case ID_REWIND:
      case ID_FAST_FORWARD:
	break;
      case ID_APP_ABOUT:
	  {
	    Dialog	    dlg(_wnd, IDD_ABOUTBOX);
	    dlg.DoModal();
	  }
	break;
      case ID_OPTIONS_PREFERENCES:
	  {
	    PreferencesDialog	dlg(_wnd);
	    dlg.DoModal();
	  }
	break;
      case ID_OPTIONS_PREVIEW:
	  {
	    PreviewSettingsDialog	dlg(_wnd);
	    dlg.DoModal();
	  }
	break;
    }
}

void
MainWindow::OnUpdate(SceneView *sender, int type, Hint *hint)
{
    switch (type) {
      case UPDATE_SELECTION:
      case UPDATE_ALL:
	UpdateToolbars();
	break;
    }
}

void
MainWindow::OnSize(int width, int height)
{
    PanedWindow::OnSize(width, height);
    if (!swIsMaximized(_wnd)) {
	swGetTotalSize(_wnd, &width, &height);
	TheApp->SetIntPreference("WindowWidth", width);
	TheApp->SetIntPreference("WindowHeight", height);
    }
}

//
// UpdateToolbars()
//
// updates all the "insert"-type buttons (eg., appearance, material, etc)
// by checking to see if the current selection contains a field which
// can hold this type.
//

void
MainWindow::UpdateToolbars()
{
    Node       *node = _scene->getSelection()->getNode();
    int		field = _scene->getSelection()->getField();
    unsigned	i;

    for (i = 1; i < 13; i++) {
	int	valid = node->findValidFieldType(buttons2[i*2]) != -1;
	swToolbarSetButtonFlags(_nodeToolbar2, i, SW_TB_DISABLED, valid ? 0 :
				SW_TB_DISABLED);
    }
    swToolbarSetButtonFlags(_nodeToolbar2, 8, SW_TB_DISABLED,
	node->findValidFieldType(NODE_TEXTURE) != -1 ? 0 : SW_TB_DISABLED);
    swToolbarSetButtonFlags(_nodeToolbar2, 9, SW_TB_DISABLED,
	node->findValidFieldType(NODE_TEXTURE) != -1 ? 0 : SW_TB_DISABLED);
    swToolbarSetButtonFlags(_nodeToolbar2, 15, SW_TB_DISABLED,
	node->findValidFieldType(NODE_AUDIO_CLIP) != -1 ? 0 : SW_TB_DISABLED);
    int	disablePaste = 1;
    if (_clipboardNode) {
	if (field == -1) {
	    field = node->findValidField(_clipboardNode);
	}
	if (field != -1) {
	    disablePaste = 0;
	}
    }
    swToolbarSetButtonFlags(_standardToolbar, 7, SW_TB_DISABLED,
			    disablePaste ? SW_TB_DISABLED : 0);
}

void
MainWindow::RefreshRecentFileMenu()
{
    int		i, n = TheApp->GetNumRecentFiles();

    // insert a placeholder (if file 1 is there)
    swInsertMenuItem(_menu, ID_FILE_MRU_FILE1, "Recent File",
    		     ID_FILE_MRU_PLACEHOLDER);
    swMenuSetFlags(_menu, ID_FILE_MRU_PLACEHOLDER, SW_MENU_DISABLED,
    		   SW_MENU_DISABLED);
    
    // delete all recent-file menus
    for (i = 0; i < 16; i++) {
	swDeleteMenuItem(_menu, ID_FILE_MRU_FILE1 + i);
    }
    
    // insert recent-file menus
    if (n > 0) {
	for (i = 0; i < n; i++) {
	    char	buf[1024];
	    int	id = ID_FILE_MRU_FILE1 + i;
	    sprintf(buf, "&%d %s", i + 1,
	    	    (const char *) TheApp->GetRecentFile(i));
	    swInsertMenuItem(_menu, ID_FILE_MRU_PLACEHOLDER, buf, id);
	}

	// remove the placeholder
	swDeleteMenuItem(_menu, ID_FILE_MRU_PLACEHOLDER);
    }
}

void MainWindow::CreateNode(const char *type)
{
    Node       *node = _scene->createNode(type);

    _scene->execute(new MoveCommand(node, NULL, -1, _scene->getRoot(), 0));
}

void MainWindow::CreateGeometryNode(const char *type)
{
    Node       *node = _scene->createNode(type);
    Node       *shape = _scene->createNode("Shape");
    Node       *transform = _scene->createNode("Transform");
    Node       *appearance = _scene->createNode("Appearance");
    Node       *material = _scene->createNode("Material");

    transform->setField(1, new MFNode(new NodeList(shape)));
    shape->setField(1, new SFNode(node));
    shape->setField(0, new SFNode(appearance));
    appearance->setField(0, new SFNode(material));
    _scene->execute(new MoveCommand(transform, NULL, -1, _scene->getRoot(), 0));
    _scene->setSelection(node);
    _scene->UpdateViews(NULL, UPDATE_SELECTION);
}

void
MainWindow::CreateElevationGrid()
{
    ElevationGridDialog   dlg(_wnd, 5, 5);
    if (dlg.DoModal() == IDOK) {
	int	width = dlg.GetWidth();
	int	depth = dlg.GetDepth();
	Node       *node = _scene->createNode("ElevationGrid");
	Node       *shape = _scene->createNode("Shape");
	Node       *transform = _scene->createNode("Transform");
	Node       *appearance = _scene->createNode("Appearance");
	Node       *material = _scene->createNode("Material");

	transform->setField(1, new MFNode(new NodeList(shape)));
	shape->setField(1, new SFNode(node));
	shape->setField(0, new SFNode(appearance));
	appearance->setField(0, new SFNode(material));
	float	   *values = new float[width * depth];
	memset(values, 0, width * depth * sizeof(float));
	node->setField(3, new MFFloat(values, width * depth));
	node->setField(9, new SFInt32(width));
	node->setField(11, new SFInt32(depth));
	delete [] values;
	_scene->execute(new MoveCommand(transform, NULL, -1, _scene->getRoot(), 0));
	_scene->setSelection(node);
	_scene->UpdateViews(NULL, UPDATE_SELECTION);
    }
}

void
MainWindow::InsertImageTexture()
{
    char	buf[FILENAME_MAX];

    buf[0] = '\0';
    if (swOpenFileDialog(_wnd, "Select",
	 "Image Files (*.gif, *.jpg)\0*.gif;*.jpg\0All Files (*.*)\0*.*\0\0",
	 buf, FILENAME_MAX)) {
	Node       *current = _scene->getSelection()->getNode();
	int	    field = current->findValidFieldType(NODE_TEXTURE);

	if (field != -1) {
	    Node    *node = _scene->createNode("ImageTexture");
	    URL		url;
	    url.FromPath(buf);
	    node->setField(0, new MFString((const char *) url));
	    _scene->execute(new MoveCommand(node, NULL, -1, current, field));
	}
    }
}

void
MainWindow::InsertNode(int fieldType, const char *type)
{
    Node       *current = _scene->getSelection()->getNode();
    int		field = current->findValidFieldType(fieldType);

    if (field != -1) {
	Node    *node = _scene->createNode(type);
	_scene->execute(new MoveCommand(node, NULL, -1, current, field));
    }
}

void MainWindow::ToggleView(int direction, SceneView *view, int id,
			    const char *name)
{
    _innerPane->TogglePane(direction, view);
    bool	flag = _innerPane->GetPane(direction) != NULL;
    swMenuSetFlags(_menu, id, SW_MENU_CHECKED, flag ? SW_MENU_CHECKED : 0);
    TheApp->SetBoolPreference(name, flag);
}

void MainWindow::ToggleToolbar(ToolbarWindow *tbWin, STOOLBAR toolbar,
			       int id, const char *name)
{
    bool visible = !swIsVisible(swToolbarGetWindow(toolbar));
    swMenuSetFlags(_menu, id, SW_MENU_CHECKED, visible ? SW_MENU_CHECKED : 0);
    tbWin->ShowToolbar(toolbar, visible);
    TheApp->SetBoolPreference(name, visible);
    Layout();
}

STOOLBAR
MainWindow::LoadToolbar(ToolbarWindow *tbWin, int id, int count,
			const int *buttons, int menuid, const char *prefName)
{
    STOOLBAR	toolbar;
    bool visible = TheApp->GetBoolPreference(prefName, true);
    toolbar = tbWin->LoadToolbar(id, count, buttons);
    tbWin->ShowToolbar(toolbar, visible);
    swMenuSetFlags(_menu, menuid, SW_MENU_CHECKED,
    		   visible ? SW_MENU_CHECKED : 0);
    return toolbar;
}

void MainWindow::SetTransformMode(TransformMode mode)
{
    _scene->setTransformMode(mode);
    swMenuSetFlags(_menu, ID_ROTATE_MODE, SW_MENU_CHECKED,
    		   mode == TM_ROTATE ? SW_MENU_CHECKED : 0);
    swMenuSetFlags(_menu, ID_MOVE_MODE, SW_MENU_CHECKED,
    		   mode == TM_TRANSLATE ? SW_MENU_CHECKED : 0);
    swMenuSetFlags(_menu, ID_SCALE_MODE, SW_MENU_CHECKED,
    		   mode == TM_SCALE ? SW_MENU_CHECKED : 0);
    swToolbarSetButtonFlags(_standardToolbar, 10, SW_TB_CHECKED,
    		   mode == TM_TRANSLATE ? SW_TB_CHECKED : 0);
    swToolbarSetButtonFlags(_standardToolbar, 11, SW_TB_CHECKED,
    		   mode == TM_ROTATE ? SW_TB_CHECKED : 0);
    swToolbarSetButtonFlags(_standardToolbar, 12, SW_TB_CHECKED,
    		   mode == TM_SCALE ? SW_TB_CHECKED : 0);
    _scene->UpdateViews(NULL, UPDATE_MODE);
}

int MainWindow::OnTimer()
{
    if (_scene->isRunning()) {
	_scene->updateTime();
	return TRUE;
    } else {
	return FALSE;
    }
}

void MainWindow::Play()
{
    if (_scene->isRunning()) {
	swKillTimer(_timer);
	_scene->stop();
    } else {
	_timer = swSetTimer(_wnd, 1, timerCB, this);
	_scene->start();
    }
    swToolbarSetButtonFlags(_vcrToolbar, 3, SW_TB_CHECKED,
    			    _scene->isRunning() ? SW_TB_CHECKED : 0);
}

void MainWindow::Stop()
{
    if (_scene->isRunning()) {
	swKillTimer(_timer);
	_scene->stop();
    }
    _scene->setRecording(false);
    swToolbarSetButtonFlags(_vcrToolbar, 3, SW_TB_CHECKED, 0);
    swToolbarSetButtonFlags(_vcrToolbar, 2, SW_TB_CHECKED, 0);
}

void MainWindow::Record()
{
    _scene->setRecording(!_scene->isRecording());
    swToolbarSetButtonFlags(_vcrToolbar, 2, SW_TB_CHECKED,
    			    _scene->isRecording() ? SW_TB_CHECKED : 0);
}

void
MainWindow::OnFileOpen()
{
    char	path[1024];

    path[0] = '\0';
    if (swOpenFileDialog(_wnd, "Open",
	     "VRML (.wrl)\0*.wrl\0All Files (*.*)\0*.*", path, 1024)) {
	OpenFileCheck(path);
    }
}

//
// OpenFileCheck() -- open a file, or display a messagebox if there's a problem
//

void
MainWindow::OpenFileCheck(const char *path)
{
    if (!TheApp->OpenFile(path)) {
	char	msg[1024];
	sprintf(msg, "%s:  %s", path, strerror(errno));
	swMessageBox(_wnd, msg, "Open", SW_MB_OK, SW_MB_WARNING);
    }
}

bool
MainWindow::SaveFile(const char *path, const char *url)
{
    FILE	*f = fopen(path, "wb");

    if (!f) {
	char	msg[1024];
	sprintf(msg, "%s:  %s", path, strerror(errno));
	swMessageBox(_wnd, msg, "Save", SW_MB_OK, SW_MB_WARNING);
	return false;
    } else {
	_scene->write(f, url, false);
	fclose(f);
	return true;
    }
}

void MainWindow::OnFileSave()
{
    const char	*path = _scene->getPath();
    if (path[0]) {
	SaveFile(path, _scene->getURL());
    } else {
	OnFileSaveAs();
    }
}

void MainWindow::OnFileSaveAs()
{
    char	path[1024];

    path[0] = '\0';
    if (swSaveFileDialog(_wnd, "Save As",
    			 "VRML (.wrl)\0*.wrl\0All Files (*.*)\0*.*",
			 path, 1024)) {
	URL		url;

	url.FromPath(path);
	if (SaveFile(path, url)) {
	    TheApp->AddToRecentFiles(path);
	    _scene->setPath(path);
	    _scene->setURL(url);
	    UpdateTitle();
	}
    }
}

void MainWindow::OnEditCut()
{
    const Path *sel = _scene->getSelection();
    Node       *node = sel->getNode();
    Node       *parent = sel->getParent();
    int		parentField = sel->getParentField();

    OnEditCopy();
    _scene->execute(new MoveCommand(node, parent, parentField, NULL, -1));
}

void MainWindow::OnEditCopy()
{
    Node *node = _scene->getSelection()->getNode();
    if (node) {
	if (_clipboardNode != NULL) _clipboardNode->unref();
	_clipboardNode = node->copy();
	_clipboardNode->ref();
    }
}

void MainWindow::OnEditPaste()
{
    Node	       *destNode = _scene->getSelection()->getNode();
    int			destField = _scene->getSelection()->getField();
    
    if (_clipboardNode != NULL) {
	if (destField == -1) {
	    destField = destNode->findValidField(_clipboardNode);
	}
	if (destField != -1) {
	    Node    *node = _clipboardNode->copy();
	    _scene->execute(new MoveCommand(node, NULL, -1,
	    				    destNode, destField));
	}
    }
}

void MainWindow::OnFilePreview()
{
    char	    path[1024];

    swGetTempPath(path, 1000);
    strcat(path, "preview.wrl");

    FILE	*f = fopen(path, "wb");

    if (!f) {
	char    msg[1024];
	sprintf(msg, "%s:  %s\n", path, strerror(errno));
	swMessageBox(_wnd, msg, "Preview", SW_MB_OK, SW_MB_WARNING);
    } else {
	URL	 fileURL;
	fileURL.FromPath(path);
	_scene->write(f, fileURL, true);
	fclose(f);
	swBrowserPreview(TheApp->GetBrowser(), path, _wnd);
    }
}

bool MainWindow::SaveModified()
{
    bool	ok = true;
    char	str[256], message[256], title[256];

    if (_scene->isModified()) {
	const char	*path = _scene->getPath();
	swSetFocus(_wnd);
	swLoadString(ID_SAVE_CHANGES, str, 256);
	swLoadString(ID_SAVE_CHANGES_TITLE, title, 256);
	if (path[0]) {
	    sprintf(message, str, path);
	} else {
	    sprintf(message, str, "Untitled");
	}
	switch (swMessageBox(_wnd, message, title, SW_MB_YESNOCANCEL,
			     SW_MB_WARNING))
	{
	  case IDYES:
	    OnFileSave();
	    break;
	  case IDCANCEL:
	    ok = false;
	    break;
	  case IDNO:
	    break;
	}
    }
    return ok;
}
