/*
 * MainWindow.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _MAIN_WINDOW_H
#define _MAIN_WINDOW_H

#ifndef _PANED_WINDOW_H
#include "PanedWindow.h"
#endif

#ifndef _SCENE_H
#include "Scene.h"
#endif

class SceneView;
class ToolbarWindow;
typedef struct SToolbar *STOOLBAR;
typedef struct STimer *STIMER;
typedef struct SMenu *SMENU;

class MainWindow : public PanedWindow {
public:
    			MainWindow(Scene *scene, SWND parent);
    virtual	       ~MainWindow();

    virtual void	OnCommand(int id);
    virtual void        OnUpdate(SceneView *sender, int type, Hint *hint);
    virtual void	OnSize(int x, int y);

    void                OnFileOpen();
    void		OnFileSave();
    void		OnFileSaveAs();
    void		OnFilePreview();
    int			OnTimer();
    bool		SaveModified();

protected:
    void		CreateNode(const char *type);
    void		CreateGeometryNode(const char *type);
    void		CreateElevationGrid();
    void		InsertNode(int fieldType, const char *type);
    void		InsertImageTexture();
    void		ToggleView(int direction, SceneView *view, int id,
    				   const char *name);
    STOOLBAR		LoadToolbar(ToolbarWindow *tbWin, int id, int count,
    				    const int *buttons, int menuid,
				    const char *prefName);
    void		ToggleToolbar(ToolbarWindow *tbWin, STOOLBAR toolbar,
    				      int id, const char *name);
    void		SetTransformMode(TransformMode mode);
    void		OnEditCut();
    void		OnEditCopy();
    void		OnEditPaste();
    void		UpdateToolbars();
    void		Play();
    void		Stop();
    void		Record();
    void		OpenFileCheck(const char *path);
    bool		SaveFile(const char *path, const char *url);
    void		RefreshRecentFileMenu();
    void		UpdateTitle();

protected:
    SMENU		_menu;
    ToolbarWindow      *_toolbarWindow;
    ToolbarWindow      *_toolbarWindow2;
    PanedWindow        *_innerPane;
    SceneView	       *_treeView;
    SceneView	       *_3DView;
    SceneView	       *_fieldView;
    SceneView	       *_graphView;
    SceneView	       *_channelView;
    STOOLBAR		_standardToolbar;
    STOOLBAR		_nodeToolbar1;
    STOOLBAR		_nodeToolbar2;
    STOOLBAR		_nodeToolbar3;
    STOOLBAR		_vcrToolbar;
    STIMER		_timer;
    Node	       *_clipboardNode;
};

#endif /* _MAIN_WINDOW_H */
