/*
 * NodeAnchor.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"
#include "NodeAnchor.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "MFString.h"
#include "SFString.h"
#include "SFVec3f.h"
#include "Field.h"	    // for FF_URL

ProtoAnchor::ProtoAnchor(Scene *scene)
  : Proto(scene, "Anchor")
{
    addEventIn(MFNODE, "addChildren");
    addEventIn(MFNODE, "removeChildren");
    addExposedField(MFNODE, "children", new MFNode(), NODE_CHILD);
    addExposedField(SFSTRING, "description", new SFString(""));
    addExposedField(MFSTRING, "parameter", new MFString());
    addExposedField(MFSTRING, "url", new MFString(), FF_URL, NULL);
    addField(SFVEC3F, "bboxCenter", new SFVec3f(0.0f, 0.0f, 0.0f));
    addField(SFVEC3F, "bboxSize", new SFVec3f(-1.0f, -1.0f, -1.0f), -1.0f);
}

NodeAnchor::NodeAnchor(Scene *scene, Proto *proto)
  : Node(scene, proto)
{
}

void
NodeAnchor::preDraw()
{
    NodeList    *children = ((MFNode *) getField(0))->getValue();

    for (int i = 0; i < children->size(); i++)
	children->get(i)->preDraw();
}

void
NodeAnchor::draw()
{
    int		i;
    NodeList   *children = ((MFNode *) getField(0))->getValue();

    for (i = 0; i < children->size(); i++)
	children->get(i)->bind();

    glPushName(0);			// field offset
    glPushName(0);
    for (i = 0; i < children->size(); i++) {
	glLoadName(i);
	children->get(i)->draw();
    }
    glPopName();
    glPopName();

    for (i = 0; i < children->size(); i++)
	children->get(i)->unbind();
}

