/*
 * NodeBox.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <assert.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeBox.h"
#include "Scene.h"
#include "Proto.h"
#include "FieldValue.h"
#include "FieldCommand.h"
#include "SFVec3f.h"
#include "Node.h"
#include "Util.h"

ProtoBox::ProtoBox(Scene *scene)
  : Proto(scene, "Box")
{
    addField(SFVEC3F, "size", new SFVec3f(2.0f, 2.0f, 2.0f), 0.0f);
}

NodeBox::NodeBox(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeBox::draw()
{
    const float	*size = ((SFVec3f *) getField(0))->getValue();

    Util::DrawBox(size[0] * 0.5f, size[1] * 0.5f, size[2] * 0.5f);
}

void
NodeBox::drawHandles()
{
    const float	*size = ((SFVec3f *) getField(0))->getValue();

    glPushMatrix();
    glScalef(size[0] * 0.5f, size[1] * 0.5f, size[2] * 0.5f);
    glPushAttrib(GL_LIGHTING);
    glDisable(GL_LIGHTING);
    glPushName(0);
    glColor3f(1.0f, 1.0f, 1.0f);
    for (int i = 0; i < 8; i++) {
	glLoadName(i);
	drawHandle(boxCorners[i]);
    }
    glPopName();
    glPopAttrib();
    glPopMatrix();
}

Vec3f
NodeBox::getHandle(int handle, int * /* constraint */, int *field)
{
    const float	*size = ((SFVec3f *) getField(0))->getValue();
    switch (handle) {
      case TRF: case TLF: case TRB: case TLB:
      case BRF: case BLF: case BRB: case BLB:
	*field = 0;
        return Vec3f(size) * Vec3f(boxCorners[handle]) * 0.5f;
      default:
	assert(0);
	return Vec3f(0.0f, 0.0f, 0.0f);
    }
}

void
NodeBox::setHandle(int handle, const Vec3f &v)
{
    switch (handle) {
      case TRF: case TLF: case TRB: case TLB:
      case BRF: case BLF: case BRB: case BLB:
	_scene->setField(this, 0, new SFVec3f(v * 2.0f * Vec3f(boxCorners[handle])));
	break;
    }
}
