/*
 * NodeCone.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "stdafx.h"

#include "NodeCone.h"
#include "Proto.h"
#include "FieldValue.h"
#include "SFFloat.h"
#include "SFBool.h"

ProtoCone::ProtoCone(Scene *scene)
  : Proto(scene, "Cone")
{
    addField(SFFLOAT, "bottomRadius", new SFFloat(1.0f), 0.0f);
    addField(SFFLOAT, "height", new SFFloat(2.0f), 0.0f);
    addField(SFBOOL, "side", new SFBool(true));
    addField(SFBOOL, "bottom", new SFBool(true));
}

NodeCone::NodeCone(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void NodeCone::draw()
{
    float bottomRadius = ((SFFloat *) getField(0))->getValue();
    float height = ((SFFloat *) getField(1))->getValue();
    int side = ((SFBool *) getField(2))->getValue();
    int bottom = ((SFBool *) getField(3))->getValue();

    GLUquadricObj   *obj = gluNewQuadric();
    if (glIsEnabled(GL_TEXTURE_2D)) gluQuadricTexture(obj, GL_TRUE);

    glPushMatrix();
    glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
    glTranslatef(0.0f, 0.0f, -height * 0.5f);
    if (side) {
	gluCylinder(obj, bottomRadius, 0.0f, height, 16, 2);
    }
    if (bottom) {
	glPushMatrix();
	glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
	gluDisk(obj, 0.0f, bottomRadius, 20, 1);
	glPopMatrix();
    }
    glPopMatrix();

    gluDeleteQuadric(obj);
}
