/*
 * NodeDirectionalLight.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeDirectionalLight.h"
#include "Scene.h"
#include "Proto.h"
#include "FieldValue.h"
#include "SFFloat.h"
#include "SFVec3f.h"
#include "SFBool.h"
#include "SFColor.h"
#include "Node.h"
#include "ExposedField.h"


ProtoDirectionalLight::ProtoDirectionalLight(Scene *scene)
  : Proto(scene, "DirectionalLight")
{
    addExposedField(SFFLOAT, "ambientIntensity", new SFFloat(0.0f), 0.0f, 1.0f);
    addExposedField(SFCOLOR, "color", new SFColor(1.0f, 1.0f, 1.0f), 0.0f, 1.0f);
    addExposedField(SFVEC3F, "direction", new SFVec3f(0.0f, 0.0f, -1.0f));
    addExposedField(SFFLOAT, "intensity", new SFFloat(1.0f), 0.0f, 1.0f);
    addExposedField(SFBOOL, "on", new SFBool(true));
}

NodeDirectionalLight::NodeDirectionalLight(Scene *scene, Proto *def)
  : Node(scene, def)
{
}


void
NodeDirectionalLight::bind()
{
    float ambientIntensity = ((SFFloat *) getField(0))->getValue();
    const float *color = ((SFColor *) getField(1))->getValue();
    const float *direction = ((SFVec3f *) getField(2))->getValue();
    float intensity = ((SFFloat *) getField(3))->getValue();
    int on = ((SFBool *) getField(4))->getValue();

    float ambientColor[4], diffuseColor[4];
    float pos[4];
    for (int i = 0; i < 3; i++) {
	ambientColor[i] = color[i] * ambientIntensity;
	diffuseColor[i] = color[i] * intensity;
	pos[i] = -direction[i];
    }
    ambientColor[3] = diffuseColor[3] = 1.0f;
    pos[3] = 0.0f;

    if (on) {
	_light = (GLenum) _scene->allocateLight();

	glLightfv(_light, GL_AMBIENT, ambientColor);
	glLightfv(_light, GL_DIFFUSE, diffuseColor);
	glLightfv(_light, GL_POSITION, pos);
	glLightfv(_light, GL_SPECULAR, diffuseColor);
	glLightf(_light, GL_SPOT_CUTOFF, 180.0f);
	glLightf(_light, GL_SPOT_EXPONENT, 0.0f);
	glLightf(_light, GL_CONSTANT_ATTENUATION, 1.0f);
	glLightf(_light, GL_LINEAR_ATTENUATION, 0.0f);
	glLightf(_light, GL_QUADRATIC_ATTENUATION, 0.0f);
	glEnable(_light);
    }
}

void NodeDirectionalLight::unbind()
{
    int on = ((SFBool *) getField(4))->getValue();

    if (on) {
	glDisable(_light);
	_scene->freeLight();
    }
}
