/*
 * NodeGroup.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeGroup.h"
#include "Proto.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "SFVec3f.h"


ProtoGroup::ProtoGroup(Scene *scene)
  : Proto(scene, "Group")
{
    addEventIn(MFNODE, "addChildren");
    addEventIn(MFNODE, "removeChildren");
    addExposedField(MFNODE, "children", new MFNode(), NODE_CHILD);
    addField(SFVEC3F, "bboxCenter", new SFVec3f(0.0f, 0.0f, 0.0f));
    addField(SFVEC3F, "bboxSize", new SFVec3f(-1.0f, -1.0f, -1.0f), -1.0f);
}

NodeGroup::NodeGroup(Scene *scene, Proto *def)
  : Node(scene, def)
{
}


void
NodeGroup::draw()
{
    int		i;
    NodeList   *children = ((MFNode *) getField(0))->getValue();

    glPushName(0);	// field offset

    for (i = 0; i < children->size(); i++)
	children->get(i)->bind();

    glPushName(0);
    for (i = 0; i < children->size(); i++) {
	glLoadName(i);
	children->get(i)->draw();
    }
    glPopName();

    for (i = 0; i < children->size(); i++)
	children->get(i)->unbind();

    glPopName();
}

void
NodeGroup::preDraw()
{
    NodeList    *children = ((MFNode *) getField(0))->getValue();

    for (int i = 0; i < children->size(); i++)
	children->get(i)->preDraw();
}
