/*
 * NodeSwitch.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeSwitch.h"
#include "Proto.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "SFInt32.h"


ProtoSwitch::ProtoSwitch(Scene *scene)
  : Proto(scene, "Switch")
{
    addExposedField(MFNODE, "choice", new MFNode(), NODE_CHILD);
    addExposedField(SFINT32, "whichChoice", new SFInt32(-1), -1.0f);
}

NodeSwitch::NodeSwitch(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeSwitch::preDraw()
{
    NodeList *choice = ((MFNode *) getField(0))->getValue();
    int whichChoice = ((SFInt32 *) getField(1))->getValue();

    if (whichChoice < 0 || whichChoice >= choice->size()) return;

    choice->get(whichChoice)->preDraw();
}

void
NodeSwitch::draw()
{
    NodeList *choice = ((MFNode *) getField(0))->getValue();
    int whichChoice = ((SFInt32 *) getField(1))->getValue();

    if (whichChoice < 0 || whichChoice >= choice->size()) return;

    glPushName(0);
    glPushName(whichChoice);
    choice->get(whichChoice)->draw();
    glPopName();
    glPopName();
}
