/*
 * NodeText.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeText.h"
#include "Proto.h"
#include "FieldValue.h"
#include "MFString.h"
#include "SFNode.h"
#include "MFFloat.h"
#include "SFFloat.h"
#include "SFString.h"
#include "SFBool.h"
#include "Scene.h"
#include "NodeFontStyle.h"
#include "FontInfo.h"

ProtoText::ProtoText(Scene *scene)
  : Proto(scene, "Text")
{
    addExposedField(MFSTRING, "string", new MFString());
    addExposedField(SFNODE, "fontStyle", new SFNode(NULL), NODE_FONT_STYLE);
    addExposedField(MFFLOAT, "length", new MFFloat(), 0.0f);
    addExposedField(SFFLOAT, "maxExtent", new SFFloat(0.0f), 0.0f);

}

NodeText::NodeText(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeText::draw()
{
    MFString	   *string = (MFString *) getField(0);
    NodeFontStyle  *fontStyle = (NodeFontStyle *) ((SFNode *) getField(1))->getValue();
//    const float	   *length = ((MFFloat *) getField(2))->getValue();
//    float	    maxExtent = ((SFFloat *) getField(3))->getValue();
    FontInfo	   *font = NULL;
    bool	    horizontal = true;
    bool	    leftToRight = true;
    float	    size = 1.0f;
    float	    spacing = 1.0f;
    bool	    topToBottom = true;

    if (fontStyle) {
	MFString       *family = (MFString *) fontStyle->getField(0);
			horizontal = ((SFBool *) fontStyle->getField(1))->getValue();
//	MFString       *justify = (MFString *) fontStyle->getField(2);
//	const char     *language = ((SFString *) fontStyle->getField(3))->getValue();
			leftToRight = ((SFBool *) fontStyle->getField(4))->getValue();
			size = ((SFFloat *) fontStyle->getField(5))->getValue();
	    		spacing = ((SFFloat *) fontStyle->getField(6))->getValue();
	const char     *style = ((SFString *) fontStyle->getField(7))->getValue();
			topToBottom = ((SFBool *) fontStyle->getField(8))->getValue();

	for (int i = 0; i < family->getSize(); i++) {
	    font = _scene->LoadGLFont(family->getValue(i), style);
	    if (font) break;
	}

    } else {
	font = _scene->LoadGLFont("SERIF", "PLAIN");
    }

    if (!font) return;	    // couldn't find the font

    glPushMatrix();
    glPushAttrib(GL_TEXTURE_BIT);

    glScalef(size, size, 1.0f);

    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);

    glEnable(GL_TEXTURE_GEN_S);
    glEnable(GL_TEXTURE_GEN_T);

    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    float	    sPlane[4] = {1.0, 0.0, 0.0, 0.0};
    float	    tPlane[4] = {0.0, 1.0, 0.0, 0.0};

    float	    ySpacing = topToBottom ? -spacing : spacing;

    for (int j = 0; j < string->getSize(); j++) {
	const char  *str = string->getValue(j);
	int	     n = strlen(str);
	if (horizontal) {
	    sPlane[3] = 0.0f;
	    glPushMatrix();
	    for (int i = 0; i < n; i++) {
		if (leftToRight) {
		    glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlane);
		    glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlane);
		    glCallList(font->displayListBase + str[i]);
		    sPlane[3] += font->kernX[str[i]];
		} else {
		    glTranslatef(-font->kernX[str[i]], 0.0f, 0.0f);
		    sPlane[3] -= font->kernX[str[i]];
		    glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlane);
		    glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlane);
		    glPushMatrix();
		    glCallList(font->displayListBase + str[i]);
		    glPopMatrix();
		}
	    }
	    glPopMatrix();
	    glTranslatef(0.0f, ySpacing, 0.0f);
	    tPlane[3] += ySpacing;
	} else {
	    tPlane[3] = 0.0f;
	    if (!leftToRight) {
		glTranslatef(-spacing * 0.5f, 0.0f, 0.0f);
		sPlane[3] += -spacing * 0.5f;
	    }
	    glPushMatrix();
	    for (int i = 0; i < n; i++) {
		if (topToBottom) {
		    float	    y = -font->kernY[str[i]];
		    if (y == 0.0) y = -1.0f;	// FIXME:  default advance?
		    glTranslatef(0.0f, y, 0.0f);
		    tPlane[3] += y;
		    glPushMatrix();
		    glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlane);
		    glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlane);
		    glCallList(font->displayListBase + str[i]);
		    glPopMatrix();
		} else {
		    glPushMatrix();
		    glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlane);
		    glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlane);
		    glCallList(font->displayListBase + str[i]);
		    glPopMatrix();
		    float	    y = font->kernY[str[i]];
		    if (y == 0.0) y = 1.0f;	// FIXME:  default advance?
		    glTranslatef(0.0f, y, 0.0f);
		    tPlane[3] += y;
		}
	    }
	    glPopMatrix();
	    if (leftToRight) {
		glTranslatef(spacing * 0.5f, 0.0f, 0.0f);
		sPlane[3] += spacing * 0.5f;
	    }
	}
    }

    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);

    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);

    glPopAttrib();
    glPopMatrix();
}
