/*
 * NodeTimeSensor.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "NodeTimeSensor.h"
#include "Proto.h"
#include "Scene.h"
#include "FieldValue.h"
#include "SFTime.h"
#include "SFBool.h"
#include "SFFloat.h"


ProtoTimeSensor::ProtoTimeSensor(Scene *scene)
  : Proto(scene, "TimeSensor")
{
    addExposedField(SFTIME, "cycleInterval", new SFTime(1.0), 0.0f);
    addExposedField(SFBOOL, "enabled", new SFBool(true));
    addExposedField(SFBOOL, "loop", new SFBool(false));
    addExposedField(SFTIME, "startTime", new SFTime(0.0));
    addExposedField(SFTIME, "stopTime", new SFTime(0.0));
    addEventOut(SFTIME, "cycleTime");
    addEventOut(SFFLOAT, "fraction_changed");
    addEventOut(SFBOOL, "isActive");
    addEventOut(SFTIME, "time");
}

NodeTimeSensor::NodeTimeSensor(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeTimeSensor::preDraw()
{
    bool	enabled = ((SFBool *) getField(1))->getValue();

    if (enabled) _scene->addTimeSensor(this);
}

void
NodeTimeSensor::setTime(double t)
{
    double	cycleInterval = ((SFTime *) getField(0))->getValue();
    bool	enabled = ((SFBool *) getField(1))->getValue();
    bool	loop = ((SFBool *) getField(2))->getValue();
    double	stopTime = ((SFTime *) getField(4))->getValue();

    if (enabled) {
	if (t > _startTime && t < stopTime) {
	    _active = true;
	}
	if (_active) {
	    if (t >= stopTime && stopTime > _startTime && !loop) {
		_active = false;
	    }
	    double	    temp = (t - _startTime) / cycleInterval;
	    double	    fraction = temp - floor(temp);
	    if (fraction == 0.0 && t > _startTime) fraction = 1.0;
	    sendEvent(6, t, new SFFloat((float) fraction));
	    sendEvent(8, t, new SFTime(t));
	}
    }
}

void
NodeTimeSensor::start(double t)
{
    bool	loop = ((SFBool *) getField(2))->getValue();
    double	startTime = ((SFTime *) getField(3))->getValue();
    double	stopTime = ((SFTime *) getField(4))->getValue();

    if (stopTime <= startTime && loop) {
	_active = true;
	_startTime = t;
    } else {
	_active = false;
	_startTime = startTime;
    }
}
