/*
 * Quaternion.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <math.h>
#include "stdafx.h"

#include "Quaternion.h"

Quaternion::Quaternion(const Vec3f &axis, float angle)
{
    float	    s = (float) sin(angle * 0.5f);

    x = axis.x * s;
    y = axis.y * s;
    z = axis.z * s;
    w = (float) cos(angle * 0.5f);
}

Quaternion
Quaternion::operator *(const Quaternion &q2) const
{
    const Quaternion   &q1 = *this;
    Quaternion	    r;

    r.x = q2.w * q1.x + q2.x * q1.w + q2.y * q1.z - q2.z * q1.y;
    r.y = q2.w * q1.y + q2.y * q1.w + q2.z * q1.x - q2.x * q1.z;
    r.z = q2.w * q1.z + q2.z * q1.w + q2.x * q1.y - q2.y * q1.x;
    r.w = q2.w * q1.w - q2.x * q1.x - q2.y * q1.y - q2.z * q1.z;

    return r;
}

Vec3f
Quaternion::operator *(const Vec3f &v) const
{
    Quaternion	    r(conj() * Quaternion(v.x, v.y, v.z, 0.0f) * *this);

    return Vec3f(r.x, r.y, r.z);
}

float
Quaternion::norm() const
{
    return (float) sqrt(x * x + y * y + z * z + w * w);
}

Quaternion
Quaternion::conj() const
{
    return Quaternion(-x, -y, -z, w);
}

Quaternion
Quaternion::operator *(float f) const
{
    return Quaternion(f * x, f * y, f * z, f * w);
}

void
Quaternion::normalize()
{
    float    rlen = 1.0f / norm();

    x *= rlen;
    y *= rlen;
    z *= rlen;
    w *= rlen;
}
