/*
 * SFNode.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <assert.h>
#include "stdafx.h"

#include "SFNode.h"
#include "Node.h"

SFNode::SFNode(Node *value)
{
    _value = value;
    if (_value) _value->ref();
}

SFNode::SFNode(const SFNode &other)
{
    if (other._value) {
	_value = other._value->copy();
	_value->ref();
    } else {
	_value = NULL;
    }
}

SFNode::~SFNode()
{
    if (_value) _value->unref();
}

void
SFNode::write(FILE *f, int indent) const
{
    if (_value) {
	_value->write(f, indent);
    } else {
	fprintf(f, "NULL\n");
    }
}

bool
SFNode::equals(const FieldValue *value) const
{
    return value->getType() == SFNODE && ((SFNode *) value)->getValue() == _value;
}

FieldValue *
SFNode::addNode(Node *node) const
{
    assert(_value == NULL);
    return new SFNode(node);
}

FieldValue *
SFNode::removeNode(Node *node) const
{
    assert(_value == node);
    return new SFNode(NULL);
}
