/*
 * SFVec3f.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <assert.h>
#include "stdafx.h"

#include "SFVec3f.h"

SFVec3f::SFVec3f(float x, float y, float z)
{
    _value[0] = x; _value[1] = y; _value[2] = z;
}

SFVec3f::SFVec3f(const Vec3f &v)
{
    _value[0] = v.x; _value[1] = v.y; _value[2] = v.z;
}

bool
SFVec3f::equals(const FieldValue *value) const
{
    if (value->getType() == SFVEC3F) {
	SFVec3f		*v = (SFVec3f *) value;
	return v->getValue()[0] == _value[0]
	    && v->getValue()[1] == _value[1]
	    && v->getValue()[2] == _value[2];
    } else {
	return false;
    }
}

void SFVec3f::write(FILE *f, int /* indent */) const
{
    fprintf(f, "%g %g %g\n", _value[0], _value[1], _value[2]);
}

void SFVec3f::setValue(int index, float value)
{
    assert(index >= 0 && index < 3);

    _value[index] = value;
}

void
SFVec3f::clamp(float min, float max)
{
    for (int i = 0; i < 3; i++) {
	if (_value[i] < min) _value[i] = min;
	if (_value[i] > max) _value[i] = max;
    }
}
